import cadquery as cq
from cq_gears import SpurGear

import copy, math

from shapely.geometry import LineString
from shapely.geometry import Point

slant_degree=15.0
thickness=20
rod_thickness=15.2
loop_thickness=1.5
wall_width=1.6
stepper_motor_size=42
lid_thicknesss = 5
offset=9
part_thinkness=20
gear_thinkness=10
large_gear_diameter=8.5

gap = stepper_motor_size * math.sin(math.radians(slant_degree)) / math.sin(math.radians(90-slant_degree))
width = math.sqrt(gap**2 + stepper_motor_size**2 - 2*gap*stepper_motor_size*math.cos(math.radians(90)))

d=20
t=7

end_x=gap+stepper_motor_size-5
end_y=-4.5
angle=40
outer_ring_diameter=rod_thickness+2*loop_thickness
center_x=end_x-math.cos(math.radians(angle))*outer_ring_diameter/2
center_y=end_y-math.sin(math.radians(angle))*outer_ring_diameter/2

def findRodInter(n_x, n_y):
    p = Point(center_x, center_y)
    c = p.buffer(rod_thickness/2).boundary
    l = LineString([(n_x, n_y), (n_x+100*math.cos(math.radians(-slant_degree)), n_y+100*math.sin(math.radians(-slant_degree)))])
    i = c.intersection(l)

    if len(i.geoms) > 1 and i.geoms[0].coords[0][0] > i.geoms[1].coords[0][0]:
        x = i.geoms[1].coords[0][0]
        y = i.geoms[1].coords[0][1]
    else:
        x = i.geoms[0].coords[0][0]
        y = i.geoms[0].coords[0][1]
    return x, y

def createOutline(ex_amount):
    base = cq.Workplane('XY').lineTo(0,offset).polarLine(stepper_motor_size, 90-slant_degree)
    base = base.hLine(width).polarLine(stepper_motor_size-5,270-slant_degree).tangentArcPoint((end_x, end_y), relative=False)

    outer_x=center_x+math.cos(math.radians(270-slant_degree))*outer_ring_diameter/2
    outer_y=center_y+math.sin(math.radians(270-slant_degree))*outer_ring_diameter/2
    base = base.threePointArc((center_x+outer_ring_diameter/2 ,center_y), (outer_x, outer_y)).polarLine(d, 180-slant_degree).vLine(t)

    n_x=base.val().endPoint().toTuple()[0]
    n_y=base.val().endPoint().toTuple()[1]
    coor_x, coor_y = findRodInter(n_x, n_y)
    last_x, last_y = findRodInter(n_x, n_y+2)

    base = base.lineTo(coor_x, coor_y)
    base = base.threePointArc((center_x+rod_thickness/2, center_y), (last_x, last_y))
    base = base.polarLine(d, 180-slant_degree).close().extrude(ex_amount)

    # Fillets
    outer_fillets=2.5
    base = base.edges('|Z and >Y').fillet(outer_fillets)
    base = base.edges('|Z and <X').fillet(outer_fillets)
    base = base.edges('|Z').edges('<<Y[-2]').fillet(5)

    return base

lid = createOutline(-lid_thicknesss).translate((0,0,part_thinkness))
base = createOutline(part_thinkness)

# Shell
copy = base.faces('>>Z[0]').workplane().center(0, -offset-loop_thickness).polarLine(stepper_motor_size, slant_degree).polarLine(-2*stepper_motor_size, 90+slant_degree).hLine(-2*stepper_motor_size).close().cutThruAll()
copy = copy.faces('+Z').workplane().center(0, -offset-loop_thickness).polarLine(2*stepper_motor_size,-slant_degree).vLine(stepper_motor_size).hLine(-2*stepper_motor_size).close().cutBlind(-lid_thicknesss)
base = base.faces('+Z').shell(-wall_width)
base = base.add(copy)
base = base.faces('<<Z[0]').workplane().center(0, offset+loop_thickness).polarLine(stepper_motor_size, -slant_degree).polarLine(stepper_motor_size, -slant_degree-3.75).vLine(2*stepper_motor_size).hLine(-2*stepper_motor_size).close().cutBlind(-lid_thicknesss)

# Stepper motor mount points
stepper_hole_width=31.0
stepper_hole_radius=stepper_hole_width/2*math.sqrt(2)
stepper_center = cq.Workplane('XY').center(0, -offset-loop_thickness).lineTo(0, offset).polarLine(stepper_motor_size/2, 90-slant_degree).polarLine(stepper_motor_size/2, -slant_degree).val().endPoint().toTuple()
base = base.faces('>>Z[0]').workplane().center(stepper_center[0], -stepper_center[1]).circle(22.5/2).pushPoints([(stepper_hole_radius*math.cos(math.radians(45+slant_degree)),stepper_hole_radius*math.sin(math.radians(45+slant_degree))), (stepper_hole_radius*math.cos(math.radians(135+slant_degree)),stepper_hole_radius*math.sin(math.radians(135+slant_degree))),(stepper_hole_radius*math.cos(math.radians(225+slant_degree)),stepper_hole_radius*math.sin(math.radians(225+slant_degree))),(stepper_hole_radius*math.cos(math.radians(315+slant_degree)),stepper_hole_radius*math.sin(math.radians(315+slant_degree)))]).circle(3/2).cutBlind(-wall_width)

lid = lid.cut(base)
copy = lid.faces('>>Z[0]').workplane().center(0, -offset-loop_thickness).polarLine(stepper_motor_size*2, slant_degree).polarLine(-2*stepper_motor_size, 90+slant_degree).hLine(-3*stepper_motor_size).close().cutThruAll()
lid = lid.faces('>Z[0]').shell(.000001-wall_width)
lid = lid.add(copy)


lid = lid.faces('>Z[1]').workplane().center(stepper_center[0], -stepper_center[1]-offset-loop_thickness).circle(5).extrude(lid_thicknesss-wall_width)
lid = lid.faces('>Z[0]').workplane().circle(5/2).cutThruAll()

gear_gap=0.2
base = base.faces('<Z[1]').workplane().center(21,20.15).circle(26.5).cutBlind(-gear_thinkness-gear_gap)
t=11/2
base = base.faces('<Z[2]').workplane().hLine(t, forConstruction=True).threePointArc((-t,-t/2),(0,t)).polarLine(t, -slant_degree).close().extrude(-lid_thicknesss+wall_width)
gear_offset=0.5
base = base.faces('<Z[2]').workplane().circle(large_gear_diameter/2-gear_offset).extrude(gear_thinkness/2)
base = base.faces('<Z[2]').workplane().circle(2).cutBlind(-part_thinkness/2+wall_width)

lid = lid.faces('>Z[0]').workplane().center(21,-20.15).hLine(t, forConstruction=True).threePointArc((-t,t/2),(0,-t)).polarLine(t, slant_degree).close().extrude(-lid_thicknesss+wall_width)
lid = lid.faces('>Z[0]').workplane().circle(large_gear_diameter/2-gear_offset).extrude(gear_thinkness/2)
lid = lid.faces('>Z[0]').workplane().circle(2).cutThruAll()
m_hole_diameter=6
m_hole_depth=3
lid = lid.faces('<Z[0]').workplane().circle(m_hole_diameter/2).cutBlind(-m_hole_depth)


mod=0.8
l_gear = cq.Workplane('XY').gear(SpurGear(module=mod,
                                          teeth_number=63,
                                          width=gear_thinkness,
                                          bore_d=large_gear_diameter))
l_gear = l_gear.translate((21+stepper_center[0],20.15+stepper_center[1]+offset+loop_thickness,(gear_thinkness+gear_offset)/2))


s_gear = cq.Workplane('XY').gear(SpurGear(module=mod,
                                          teeth_number=11,
                                          width=gear_thinkness,
                                          bore_d=4.5))
s_gear = s_gear.translate((stepper_center[0],stepper_center[1]+offset+loop_thickness,(gear_thinkness+gear_offset)/2))

# Render
show_object(base, name='Follow Focus Base')
show_object(lid, name='Follow Focus Lid')
show_object(l_gear, name='Follow Focus Large Gear')
show_object(s_gear, name='Follow Focus Small Gear')

